
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <linux/malloc.h>
#include <comedi_module.h>

#define NI_SIZE 0x20

static char pnp_magic[]={
	0x00,0x00,0x6a,0xb5,0xda,0xed,0xf6,0xfb,
	0x7d,0xbe,0xdf,0x6f,0x37,0x1b,0x0d,0x86,
	0xc3,0x61,0xb0,0x58,0x2c,0x16,0x8b,0x45,
	0xa2,0xd1,0xe8,0x74,0x3a,0x9d,0xce,0xe7,
	0x73,0x39,0 };

#define PNP_CMD 0x479
#define PNP_ADR 0x279
#define PNP_RD 0x203

#define DIO_Control_Register 11
#define DIO_Output_Register 10
#define DIO_Input_Register 7

#define NI_WIND_ADDR 0
#define NI_WIND_DATA 2

#define windowed_out(a,b,c) (outw((b),(c)+NI_WIND_ADDR),outw((a),(c)+NI_WIND_DATA))
#define windowed_in(b,c) (outw((b),(c)+NI_WIND_ADDR),inw((c)+NI_WIND_DATA))

typedef struct{
	int dio;
}ni_private;
#define devpriv ((ni_private *)dev->private)

comedi_devinit ni_init;
static int ni_dio(comedi_device *dev,comedi_trig *ad);
static int ni_rem(comedi_device *dev);
static int pnp_init(int iobase);

static int ni_dio(comedi_device *dev,comedi_trig *it)
{
	comedi_sample s;

	switch(it->io){
	case COMEDI_IO_INPUT:
		s.data=it->data=
			windowed_in(DIO_Input_Register,dev->iobase)&0xff;
		s.chan=it->chan;
	
		report_sample(dev,&s);
	
		return 0;
	case COMEDI_IO_OUTPUT:
		windowed_out(it->data&0xff,DIO_Output_Register,dev->iobase);
		return 0;
	default:
		return -EINVAL;
	}
}

static int ni_sp(comedi_device *dev,comedi_param *it)
{
	switch(it->pnum){
	case COMEDI_IOBITS:
		if(it->pval&~0xff)return -EINVAL;
		devpriv->dio=it->pval;
		windowed_out(devpriv->dio,DIO_Control_Register,dev->iobase);
		changeparam(&dev->chinfo[it->chan].paramlist,COMEDI_IOBITS,0,
			it->pval);
		return 0;
	default:
		return -EINVAL;
	}
}

int ni_init(comedi_device *dev,comedi_devconfig *it)
{
	int i;

	if(strcmp("ni",it->name))
		return 0;

	printk("comedi%d: ni: 0x%04x ",dev->minor,it->iobase);
	if(check_region(it->iobase,NI_SIZE)<0){
		printk("I/O port conflict\n");
		return -EIO;
	}
	request_region(it->iobase,NI_SIZE,"ni");
	dev->iobase=it->iobase;
	dev->iosize=NI_SIZE;
	dev->irq=0;

	dev->chan=1;

	devpriv=kmalloc(sizeof(ni_private)+dev->chan*sizeof(comedi_chinfo),
		GFP_KERNEL);
	if(!devpriv){
		release_region(dev->iobase,dev->iosize);
		return -ENOMEM;
	}
	memset(devpriv,0,sizeof(ni_private)+dev->chan*sizeof(comedi_chinfo));

	dev->chinfo=(void *)(devpriv+1);
	for(i=0;i<dev->chan;i++){
		dev->chinfo[i].trig=ni_dio;
		dev->chinfo[i].sp=ni_sp;
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_BITS,COMEDI_RDONLY,8);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_AD,COMEDI_RDONLY,COMEDI_AD_DIGITAL);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IO,COMEDI_RDONLY,COMEDI_IO_BIDIR);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITGROUP,COMEDI_RDONLY,1);
		addparam(&dev->chinfo[i].paramlist,
			COMEDI_IOBITS,COMEDI_RDWR,0xff&COMEDI_IOBITS_INPUT);
	}
	devpriv->dio=0;
	windowed_out(devpriv->dio,DIO_Control_Register,dev->iobase);

	comedi_initbuf(&dev->rdbuf);

	dev->rem=ni_rem;

	return 1;
}

#if 0
/* this is not used, since it doesn't seem to work.  use isapnptools
instead. */
static int pnp_init(int iobase)
{
	int i;

	/* initiate board */
	for(i=0;i<34;i++)
		outb(pnp_magic[i],PNP_ADR);

	/* reset pnp registers */
	outb(0x02,PNP_ADR);
	outb(0x07,PNP_CMD);
	
	/* initiate board */
	for(i=0;i<34;i++)
		outb(pnp_magic[i],PNP_ADR);

	/* set pnp isolation state */
	outb(0x03,PNP_ADR);
	outb(0x00,PNP_CMD);
	
	/* set pnp configuration state */
	outb(0x06,PNP_ADR);
	outb(0x01,PNP_CMD);
	
	/* assign pnp base address */
	
	outb(0x60,PNP_ADR);
	outb((iobase>>8)&0xff,PNP_CMD);
	outb(0x61,PNP_ADR);
	outb(iobase&0xff,PNP_CMD);
	
	/* assign pnp data port address */

	outb(0x00,PNP_ADR);
	outb(PNP_RD>>2,PNP_CMD);
	
	/* verify base address */
	
	outb(0x60,PNP_ADR);
	i=inb(PNP_RD)<<8;
	outb(0x61,PNP_ADR);
	i|=inb(PNP_RD);
	
printk("i=0x%x ",i);
	if(i!=iobase)return -EIO;
	
	/* activate board */

	outb(0x03,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x07,PNP_ADR);
	outb(0x00,PNP_CMD);

	outb(0x30,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x07,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x30,PNP_ADR);
	outb(0x01,PNP_CMD);

	outb(0x02,PNP_ADR);
	outb(0x02,PNP_CMD);
	
	return 0;
}
#endif


static int ni_rem(comedi_device *dev)
{
	int i;

	printk("comedi%d: ni: remove\n",dev->minor);
	
	for(i=0;i<dev->chan;i++)
		delparamlist(dev->chinfo[i].paramlist);
	kfree(devpriv);
	release_region(dev->iobase,dev->iosize);

	comedi_freebuf(&dev->rdbuf);

	return 0;
}

